#!/usr/bin/env python

import sys
import os

from eis1600.miu_handling import disassembling


if __name__ == "__main__":
    try:
        infile_name = sys.argv[1]
    except IndexError:
        print('Pass in a <uri.EIS1600> file to begin')
        sys.exit()

    path, uri = os.path.split(infile_name)
    uri, ext = os.path.splitext(uri)
    print(f'Disassemble {uri + ext} and storing MUI files in {path + "/" + uri + "/"}')
    disassembling.disassemble_text('.' + path + '/' + uri, uri)

    print('Done')



from eis1600.miu_handling.yml_handling import extract_yml_header_and_text


def reassemble_text(file_path, uri):
    ids = ['header', 'preface']
    with open(file_path + '.IDs', 'r', encoding='utf-8') as ids_file:
        ids.extend([line[:-1] for line in ids_file.readlines()])

    with open(file_path + '.EIS1600', 'w', encoding='utf-8') as text_file:
        with open(file_path + '.YMLDATA.yml', 'w', encoding='utf-8') as yml_data:
            for i, mui_id in enumerate(ids):
                mui_file_path = file_path + '/' + uri + '.' + mui_id + '.EIS1600'
                yml_header, text = extract_yml_header_and_text(mui_file_path, mui_id, i == 0)
                text_file.write(text)
                yml_data.write(yml_header)



#!/usr/bin/env python

import sys
import os
from argparse import ArgumentParser, Action, RawDescriptionHelpFormatter
from glob import iglob
from multiprocessing import Pool

from eis1600.markdown.methods import convert_to_eis1600


class CheckFileEndingAction(Action):
    def __call__(self, parser, namespace, input_arg, option_string=None):
        if os.path.isfile(input_arg):
            filepath, fileext = os.path.splitext(input_arg)
            if fileext != 'mARkdown':
                parser.error('You need to input a mARkdown file')
            else:
                setattr(namespace, self.dest, input_arg)
        else:
            setattr(namespace, self.dest, input_arg)


if __name__ == '__main__':

    arg_parser = ArgumentParser(prog=sys.argv[0], formatter_class=RawDescriptionHelpFormatter,
                                         description='''Script to convert mARkdown file(s) to EIS1600 file(s).
-----
Give a single mARkdown file as input
or 
Give an input AND an output directory for batch processing.''')
    arg_parser.add_argument('-v', '--verbose', action='store_true')
    arg_parser.add_argument('input', type=str,
                            help='MARkdown file to process or input directory with mARkdown files to process if an output directory if also given',
                            action=CheckFileEndingAction)
    arg_parser.add_argument('output', type=str, nargs='?',
                            help='Optional, if given batch processes all files from the input directory to the output directory')
    args = arg_parser.parse_args()

    verbose = args.verbose

    if args.input and not args.output:
        infile = args.input
        convert_to_eis1600(infile, None, verbose)

    elif args.output:
        input_dir = args.input
        output_dir = args.output
        print(f'Convert mARkdown files from {input_dir}, save results to {output_dir}')

        infiles = iglob(input_dir + '/*.mARkdown')
        params = [(infile, output_dir, verbose) for infile in infiles]

        with Pool() as p:
            p.starmap_async(convert_to_eis1600, params).get()
    else:
        print(
            'Pass in a <uri.mARkdown> file to process a single file or enter an input and output directory to use batch processing')
        sys.exit()

    print('Done')



from pathlib import Path

from eis1600.miu_handling.yml_handling import create_yml_header
from eis1600.miu_handling.re_patterns import HEADER_END_PATTERN, UID_PATTERN


def disassemble_text(file_path, uri):
    eis_file = file_path + '.EIS1600'
    ids_file = file_path + '.IDs'
    mui_dir = Path(file_path + '/')
    uid = ''
    mui_text = ''

    mui_dir.mkdir(exist_ok=True)
    mui_uri = mui_dir.__str__() + '/' + uri + '.'

    with open(eis_file, 'r', encoding='utf8') as text:
        with open(ids_file, 'w', encoding='utf8') as ids_tree:
            for text_line in iter(text):
                if HEADER_END_PATTERN.match(text_line):
                    uid = 'header'
                    mui_text += text_line
                    with open(mui_uri + uid + '.EIS1600', 'w', encoding='utf8') as mui_file:
                        mui_file.write(mui_text + '\n')
                    mui_text = ''
                    uid = 'preface'
                    next(text)  # Skip empty line after header
                elif UID_PATTERN.match(text_line):
                    with open(mui_uri + uid + '.EIS1600', 'w', encoding='utf8') as mui_file:
                        mui_file.write(mui_text)
                    uid = UID_PATTERN.match(text_line).group('UID')
                    ids_tree.write(uid + '\n')
                    mui_text = create_yml_header()
                    mui_text += text_line
                else:
                    mui_text += text_line
            # last MUI needs to be written to file when the for-loop is finished
            with open(mui_uri + uid + '.EIS1600', 'w', encoding='utf8') as mui_file:
                mui_file.write(mui_text)



from eis1600.miu_handling.re_patterns import MUI_HEADER_PATTERN, NEWLINES_PATTERN
from importlib.resources import read_text


yml_template = read_text('eis1600.mui_handling', 'yaml_template.yml')


def create_yml_header():
    yml_header = yml_template
    # TODO: populate template with MUI related information
    return yml_header + '\n'


def extract_yml_header_and_text(mui_file, mui_id, is_header):
    with open(mui_file, 'r', encoding='utf-8') as file:
        text = ''
        mui_yml_header = ''
        for line in iter(file):
            if MUI_HEADER_PATTERN.match(line):
                # Omit the #MUI#Header# line as it is only needed inside the MUI.EIS1600 file, but not in YMLDATA.yml
                next(file)
                line = next(file)
                mui_yml_header = '#' + mui_id + '\n---\n'
                while not MUI_HEADER_PATTERN.match(line):
                    mui_yml_header += line
                    line = next(file)
                # Skip empty line after #MUI#Header#
                next(file)
            else:
                text += line
            # Replace new lines which separate YAML header from text
            if not is_header:
                text, n = NEWLINES_PATTERN.subn('\n\n', text)
        return mui_yml_header, text


def update_yml_header():
    # TODO: update YAML header with MUI related information from automated analyses and manual tags
    pass



import re


UID = r'#\$(?P<UID>\d{12})\$#?\s'
UID_PATTERN = re.compile(UID)
HEADER_END_PATTERN = re.compile(r'(#META#Header#End#)')

MUI_HEADER_PATTERN = re.compile(r'#MUI#Header#')

# Fix mARkdown files
SPACES_PATTERN = re.compile(r' +')
NEWLINES_PATTERN = re.compile(r'\n{3,}')
SPACES_AFTER_NEWLINES_PATTERN = re.compile(r'\n +')
POETRY_PATTERN = re.compile(r'(%~% [^\n]+\n)\n([^\n]+ %~%)')
BELONGS_TO_PREV_PARAGRAPH_PATTERN = re.compile(r'\n(.{1,10})\n')



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='eis1600',
      version='0.2.1',
      description='EIS1600 project tools and utilities',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/EIS1600/eis1600-pkg',
      author='Lisa Mischer',
      author_email='mischer.lisa@gmail.com',
      license='MIT License',
      packages=['eis1600',
                'eis1600.miu_handling',
                'eis1600.markdown'],
      scripts=['eis1600/bin/disassemble_into_miu_files.py',
               'eis1600/bin/reassemble_from_miu_files.py',
               'eis1600/bin/convert_mARkdown_to_EIS1600.py'],
      package_data={'eis1600.miu_handling': ['yaml_template.yml']},
      python_requires='>=3.7',
      classifiers=['Programming Language :: Python :: 3',
                   'License :: OSI Approved :: MIT License',
                   'Operating System :: OS Independent',
                   'Development Status :: 1 - Planning',
                   'Intended Audience :: Science/Research']
      )



from textwrap import wrap
from random import randint
from os.path import split, splitext

from eis1600.miu_handling.re_patterns import HEADER_END_PATTERN, SPACES_PATTERN, NEWLINES_PATTERN, POETRY_PATTERN, \
    BELONGS_TO_PREV_PARAGRAPH_PATTERN, SPACES_AFTER_NEWLINES_PATTERN


def generate12ids(iterations):
    ids = []
    for i in range(0, iterations):
        ids.append('$%s$' % str(randint(400000000000, 999999999999)))
        # input(ids)
    ids = list(set(ids))
    # print('ids: {:,}'.format(len(ids)))
    return ids


def wrap_paragraph(paragraph, len_symb):
    wrapped = '\n'.join(wrap(paragraph, len_symb))
    return wrapped


def convert_to_eis1600(infile, output_dir, verbose):
    path, uri = split(infile)
    uri, ext = splitext(uri)
    if output_dir:
        outfile = output_dir + '/' + uri + '.EIS1600'
    else:
        outfile = '.' + path + '/' + uri + '.EIS1600'
    if verbose:
        print(f'Convert {uri} from mARkdown to EIS1600 file')


    with open(infile, 'r', encoding='utf8') as infileh:
        text = infileh.read()

    header_and_text = HEADER_END_PATTERN.split(text)
    header = header_and_text[0] + header_and_text[1]
    text = header_and_text[2]

    ids = generate12ids(3000000)

    # fix
    text = text.replace('~\n', '\n')
    text = text.replace('\n~~', ' ')

    # text = re.sub(r'(#~:\w+: *)', r'\n\1\n\n', text)

    text = SPACES_PATTERN.sub(' ', text)

    text = text.replace('\n###', '\n\n###')
    text = text.replace('\n# ', '\n\n')

    text = NEWLINES_PATTERN.sub('\n\n', text)

    # fix poetry
    text = POETRY_PATTERN.sub(r'\1\2', text)
    text = POETRY_PATTERN.sub(r'\1\2', text)
    text = POETRY_PATTERN.sub(r'\1\2', text)

    text = text.split('\n\n')

    text_updated = []

    counter = 0
    for m in text:
        counter += 1
        if m.startswith('### '):
            m = m.replace('### ', '#%s ' % ids[counter])
            text_updated.append(m)
        elif '%~%' in m:
            m = '%s ::POETRY:: ~\n' % ids[counter] + m
            text_updated.append(m)
        else:
            m = wrap_paragraph(m, 60)
            m = '%s ::UNDEFINED:: ~\n' % ids[counter] + m
            text_updated.append(m)

    text = '\n\n'.join(text_updated)
    text, n = BELONGS_TO_PREV_PARAGRAPH_PATTERN.subn(r' \1\n', text)

    # spaces
    text, n = SPACES_AFTER_NEWLINES_PATTERN.subn('\n', text)

    # reassemble text
    final = header + '\n\n' + text

    with open(outfile, 'w', encoding='utf8') as outfileh:
        outfileh.write(final)
